package de.uni_mannheim.app.context;

import javax.microedition.lcdui.Displayable;

import org.kalmeo.util.frame.Frame;

import de.uni_mannheim.app.business.Job;
import de.uni_mannheim.app.controllers.Controller;
import de.uni_mannheim.app.gui.AtCustomerView;
import de.uni_mannheim.app.gui.InCarDrivingView;
import de.uni_mannheim.app.gui.InCarView;
import de.uni_mannheim.app.gui.JobAssignedView;
import de.uni_mannheim.app.gui.JobDetailsView;
import de.uni_mannheim.app.gui.JobPendingView;
import de.uni_mannheim.app.gui.MessagePopupAlert;
import de.uni_mannheim.app.gui.NoJobView;
import de.uni_mannheim.app.gui.UpdateTimeView;
import de.uni_mannheim.app.gui.WorkingView;
import de.uni_mannheim.app.gui.YesNoMessageAlert;
import de.uni_mannheim.aware.util.DateUtil;

public class ViewContext implements Context {
	private static ViewContext instance;
	
	// References
	private ControllerContext controllerContext;	
	
	// Special Links
	private Frame currentView; // TODO --> what if context changed while dialog open
	private Frame backView;		
	
	// Views
	//private SplashScreenView splashScreenView;
	private AtCustomerView atCustomerView;
	private InCarDrivingView inCarDrivingView;
	private InCarView inCarView; 
	private JobAssignedView jobAssignedView;
	private JobDetailsView jobDetailsView;
	private JobPendingView jobPendingView;
	private MessagePopupAlert messagePopupView;	
	private NoJobView noJobView;
	private UpdateTimeView updateTimeView;	
	private WorkingView workingView;
	private YesNoMessageAlert yesNoMessageView;
		
	private ViewContext() {
		// Fressen!
	}
	
	public static ViewContext getInstance() {
		if (instance == null)
			instance = new ViewContext();
		
		return instance;
	}

	public void create() {
		atCustomerView = new AtCustomerView();
		inCarDrivingView = new InCarDrivingView();
		inCarView = new InCarView();
		jobAssignedView = new JobAssignedView();
		jobDetailsView = new JobDetailsView();
		jobPendingView = new JobPendingView();
		messagePopupView = new MessagePopupAlert();
		noJobView = new NoJobView();
		updateTimeView = new UpdateTimeView();
		workingView = new WorkingView();		
		yesNoMessageView = new YesNoMessageAlert();
	}
	 
	public void inject() {
		// References
		controllerContext = ControllerContext.getInstance();
		 
		// Add controllers to views
		atCustomerView.setController(controllerContext.getAtCustomerController());
		inCarDrivingView.setController(controllerContext.getInCarDrivingController());
		inCarView.setController(controllerContext.getInCarController());
		jobAssignedView.setController(controllerContext.getJobAssignedController());
		jobDetailsView.setController(controllerContext.getJobDetailsController());
		jobPendingView.setController(controllerContext.getJobPendingController());
		messagePopupView.setController(controllerContext.getMessagePopupController());
		noJobView.setController(controllerContext.getNoJobController());
		updateTimeView.setController(controllerContext.getUpdateTimeController());
		workingView.setController(controllerContext.getWorkingController());
		yesNoMessageView.setController(controllerContext.getYesNoMessageController());
	}
	
	public void activate() {
		
	}
	
	public synchronized void updateJobDetails(Job j) {

		/*String description = j.			
			"Customer: "+j.getCustomer()+"\n"+
			"Address: "+j.getAddress()+"\n"+
			"Details: "+j.getDetails()+"\n";*/
		
		getJobDetailsView().setJob(j.getTitle());
		getJobDetailsView().setDescription(j.getSummary());
		getJobDetailsView().setCustomer(j.getCustomer());
		getJobDetailsView().setLocation(j.getAddress());
		
		getJobAssignedView().setStartTime(DateUtil.dateToTimeString(j.getEstimatedStartTime()));
		getJobAssignedView().setDescription(j.getSummary());
							
		getJobPendingView().setDescription(j.getSummary());
		
		getInCarView().setStartTime(DateUtil.dateToTimeString(j.getEstimatedStartTime()));
		
		getInCarDrivingView().setStartTime(DateUtil.dateToTimeString(j.getEstimatedStartTime()));			
	}
	
	public AtCustomerView getAtCustomerView() {
		return atCustomerView;
	}

	public void setAtCustomerView(AtCustomerView atCustomerView) {
		this.atCustomerView = atCustomerView;
	}

	public Frame getCurrentView() {
		return currentView;
	}

	public void setCurrentView(Frame currentView) {
		this.currentView = currentView;
	}

	public Frame getBackView() {
		return backView;
	}

	public void setBackView(Frame backView) {
		this.backView = backView;
	}

	public InCarDrivingView getInCarDrivingView() { 
		return inCarDrivingView;
	}

	public void setInCarDrivingView(InCarDrivingView inCarDrivingView) {
		this.inCarDrivingView = inCarDrivingView;
	}

	public InCarView getInCarView() {
		return inCarView;
	}

	public void setInCarView(InCarView inCarView) {
		this.inCarView = inCarView;
	}

	public JobAssignedView getJobAssignedView() {
		return jobAssignedView;
	}

	public void setJobAssignedView(JobAssignedView jobAssignedView) {
		this.jobAssignedView = jobAssignedView;
	}

	public JobDetailsView getJobDetailsView() {
		return jobDetailsView;
	}

	public void setJobDetailsView(JobDetailsView jobDetailsView) {
		this.jobDetailsView = jobDetailsView;
	}

	public JobPendingView getJobPendingView() {
		return jobPendingView;
	}

	public void setJobPendingView(JobPendingView jobPendingView) {
		this.jobPendingView = jobPendingView;
	}

	public MessagePopupAlert getMessagePopupView() {
		return messagePopupView;
	}

	public void setMessagePopupView(MessagePopupAlert messagePopupView) {
		this.messagePopupView = messagePopupView;
	}

	public UpdateTimeView getUpdateTimeView() {
		return updateTimeView;
	}

	public void setUpdateTimeView(UpdateTimeView updateTimeView) {
		this.updateTimeView = updateTimeView;
	}

	public WorkingView getWorkingView() {
		return workingView;
	}

	public void setWorkingView(WorkingView workingView) {
		this.workingView = workingView;
	}

	public YesNoMessageAlert getYesNoMessageView() {
		return yesNoMessageView;
	}

	public void setYesNoMessageView(YesNoMessageAlert yesNoMessageView) {
		this.yesNoMessageView = yesNoMessageView;
	}

	public NoJobView getNoJobView() {
		return noJobView;
	}

	public void setNoJobView(NoJobView noJobView) {
		this.noJobView = noJobView;
	}

	
}
