package de.uni_mannheim.app.network;

import java.util.Date;
import java.util.Random;

import org.json.me.JSONObject;

import de.uni_mannheim.aware.api.SensorFactory;

/**
 * Command to be send over the network
 * @author Moritz Wächter, Philipp Heckel
 *
 */
public abstract class NetCommand{
	protected static String SERVER_ROOT_URL = 
		"http://vsrv28.informatik.uni-mannheim.de:8080";
		//(SensorFactory.getPlatform() != SensorFactory.PLATFORM_NOKIA) ? "http://localhost:8080"	: "http://vsrv28.informatik.uni-mannheim.de:8080";
	
	public static final long DEFAULT_EXPIRY_TIME = 100000;
	
	private long id;
	private String method;
	private String path;
	private Date expiryDate;

	public NetCommand(String method, String path) {
		this(method, path, new Date(System.currentTimeMillis()+DEFAULT_EXPIRY_TIME));
	}
	
	public NetCommand(String method, String path, Date expiryDate) {
		this.id = new Random().nextLong();
		this.method = method;
		this.path = path;
		this.expiryDate = expiryDate;		
	}
	
	public long getId() {
		return id;
	}
	
	public String getMethod() {
		return method;
	}

	public void setMethod(String method) {
		this.method = method;
	}
	
	public String getPath() {
		return path;
	}

	public void setPath(String path) {
		this.path = path;
	}
	
	public Date getExpiryDate() {
		return expiryDate;
	}
	
	public void setExpiryDate(Date expiryDate) {
		this.expiryDate = expiryDate;
	}		
		
	public void notifyUnsucessful(){
		//TODO
	}
	
	/* @Override */	
	public boolean equals(Object obj) {
		if (!(obj instanceof NetCommand)) 
			return false;
		
		return ((NetCommand) obj).getId() == getId();
	}
	
	/**
	 * Can be overridden if the request has a body.
	 */
	public JSONObject toJSON() {
		return null;
	}
	
	/**
	 * Can be overriden if the response needs to be parsed.
	 */
	public Object parseResponse(JSONObject object) {
		return null;
	}
	
}
