package de.uni_mannheim.app.network;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Vector;

import javax.microedition.*;

import org.json.me.JSONObject;

/**
 * 
 * @author Moritz Wächter, Philipp Heckel
 *
 */
public class NetQueue implements Runnable {
	private static NetQueue instance;

	private Vector queue;
	private Hashtable/*<NetCommand,ResponseListener>*/ responseListeners;
	private Sender sender;
	private volatile boolean shuttingDown;
	
	private NetQueue() {
		this.queue = new Vector();
		this.responseListeners = new Hashtable();
		this.sender = new Sender();
		this.shuttingDown = false;
		
		new Thread(this).start();		
	}
	
	public static NetQueue getInstance() {
		if (instance != null)
			return instance;
		
		instance = new NetQueue();
		return instance;
	}
	
	public void send(NetCommand cmd) {
		System.out.println("aa");
		send(cmd, null);
	}
	
	public synchronized void send(NetCommand cmd, ResponseListener listener) {
		queue.addElement(cmd);	
		
		System.out.println("[NetQueue] Adding command "+cmd);
		System.out.println("[NetQueue] Queue size: " + queue.size());
		
		if (listener != null)
			responseListeners.put(cmd, listener);
		//removeDuplicates(cmd);
		
		// Must have the monitor for 'this' !
		notify();
	}
	
	public synchronized void shutdown() {
		this.shuttingDown = true;		
	}	
	
	/*private void removeExpired() {
		synchronized (queue) {
			while (queue.size() > 0)
			for(int i=0; i<=queue.size() ;i++){
				NetCommand command = queue.get(i); 
				for(int j=i+1; j<queue.size();j++){
					if(queue.get(j).getClass().isInstance(command))
						queue.remove(j);
					}
				if(command.getExpiryDate()>= System.currentTimeMillis()){
					command.notifyUnsucessful();
					queue.remove(i);
				}
			}
		}
	}*/
	
	private synchronized NetCommand getNext() {
		if (queue.size() == 0) {
			System.out.println("[NetQueue] Queue empty. Waiting.");

			try { wait(); } 
			catch (InterruptedException e) { }	

			// We waited! Now let's check again.
			//take();
		}
		
		// Pick first command
		return (NetCommand) queue.elementAt(0);
	}
	
	public void run() {
		while (!shuttingDown) {
			// Pick a command (or wait until one is there) 
			NetCommand cmd = getNext(); // blocks!
							
			// Try to send it
			System.out.println("[NetQueue] Processing command "+cmd);
			
			boolean success = false;			
			JSONObject responseJSON = null;
			Object responseObject = null;

			try {
				responseJSON = sender.send(cmd.getPath(), cmd.toJSON(), cmd.getMethod());
								
				// Response Object is wrong, thus, return false.
				if(responseJSON == null)
					throw new IOException("Invalid response object: null");				
				
				// Parse and process (needed to verify success!)
				responseObject = cmd.parseResponse(responseJSON);
				System.out.println("[NetQueue] Response: "+responseObject);
				
				// Notify listener (if any)
				Object listener = responseListeners.remove(cmd);
	
				if (listener != null) {
					success = ((ResponseListener) listener).processResponse(responseObject);
					
					if (success) System.out.println("[NetQueue] Listener said success: "+listener);
					else System.out.println("[NetQueue] Listener said FAILURE: "+listener);
				}
				else {
					System.out.println("[NetQueue] No listeners. Success.");
					success = true;
				}
			} 
			catch (Exception e) {
				System.err.println("[NetQueue] ERROR: "+e);
				e.printStackTrace();
				success = false;
			}

			// Retry if failure!
			if (!success) {
				System.out.println("[NetQueue] Sending FAILED. Sleeping.");
				try { Thread.sleep(1000); }
				catch (InterruptedException e) { }
				
				continue;
			}

			// Success!
			synchronized (this) {
				queue.removeElementAt(0);
				//System.out.println("Received! QueueSize: " + queue.size());
			}								
		}
		
		System.out.println("[NetQueue] Shutdown called.");
		
		synchronized (this) {
			instance = null;
		}
	}
}
