package de.uni_mannheim.app.network.commands;

import java.io.IOException;
import java.util.Date;

import javax.microedition.io.HttpConnection;

import org.json.me.JSONException;
import org.json.me.JSONObject;
import de.uni_mannheim.app.business.*;
import de.uni_mannheim.app.network.NetCommand;
import de.uni_mannheim.app.network.Sender;

public class GetWorkerContextCommand extends NetCommand {	
	public GetWorkerContextCommand(long workerId) {
		super(HttpConnection.GET, SERVER_ROOT_URL+"/resources/app/worker/"+workerId+"/context");
	}
	
	/* @Override */
	public Object parseResponse(JSONObject object) {		
		Job currentjob = null;
		Job nextjob = null;
		Status status = null;

		// @Lars Bugfix: Every exception lead to non-initialized data
		JSONObject currentJobJSON = null;
		JSONObject nextJobJSON = null;
		
		try{
			currentJobJSON = object.getJSONObject("currentJob");
		}catch(JSONException e){
			// can happen as there may not be a current job
		}
		
		try{
			nextJobJSON = object.getJSONObject("nextJob");
		}catch(JSONException e){
			// should not happen
			System.err.println("Received an empty next job");
		}
		
		try {
			// Current job
			if (currentJobJSON != null) {
				currentjob = new Job();
						
				currentjob.setId(currentJobJSON.getLong("id"));
				currentjob.setActualDuration(currentJobJSON.getInt("actualDuration"));
				currentjob.setActualStartTime(new Date(currentJobJSON.getLong("actualStartTime")));
				currentjob.setAddress(currentJobJSON.getString("address"));
				currentjob.setLongitude(currentJobJSON.getDouble("longitude"));
				currentjob.setLatitude(currentJobJSON.getDouble("latitude"));
				currentjob.setCustomer(currentJobJSON.getString("customer"));
				currentjob.setDetails(currentJobJSON.getString("details"));
				currentjob.setEstimatedDuration(currentJobJSON.getInt("estimatedDuration"));
				currentjob.setEstimatedStartTime(new Date(currentJobJSON.getLong("estimatedStartTime")));
				currentjob.setSummary(currentJobJSON.getString("summary"));
				currentjob.setTitle(currentJobJSON.getString("title"));
			}
			
			// Next job 
			
			
			if (nextJobJSON != null) {
				nextjob = new Job();
				
				nextjob.setId(nextJobJSON.getLong("id"));
				nextjob.setActualDuration(nextJobJSON.getInt("actualDuration"));
				nextjob.setActualStartTime(new Date(nextJobJSON.getLong("actualStartTime")));
				nextjob.setAddress(nextJobJSON.getString("address"));
				nextjob.setLongitude(nextJobJSON.getDouble("longitude"));
				nextjob.setLatitude(nextJobJSON.getDouble("latitude"));			
				nextjob.setCustomer(nextJobJSON.getString("customer"));
				nextjob.setDetails(nextJobJSON.getString("details"));
				nextjob.setEstimatedDuration(nextJobJSON.getInt("estimatedDuration"));
				nextjob.setEstimatedStartTime(new Date(nextJobJSON.getLong("estimatedStartTime")));
				nextjob.setSummary(nextJobJSON.getString("summary"));
				nextjob.setTitle(nextJobJSON.getString("title"));
			}
			
			status = new Status(object.getInt("status"));
		}
		catch (JSONException e) {
			System.err.println("errr: "+e);
			currentjob = null;
			nextjob = null;
			status = Status.UNKNOWN;
		}
		
		return new WorkerContext(currentjob, nextjob, status);
	}
	
	public class WorkerContext {
		private Job currentJob;
		private Job nextJob;
		private Status status;
		
		public WorkerContext(Job currentJob, Job nextJob, Status status) {
			this.currentJob = currentJob;
			this.nextJob = nextJob;
			this.status = status;
		}
		
		public Job getCurrentJob() {
			return currentJob;
		}
		
		public void setCurrentJob(Job currentJob) {
			this.currentJob = currentJob;
		}
		
		public Job getNextJob() {
			return nextJob;
		}
		
		public void setNextJob(Job nextJob) {
			this.nextJob = nextJob;
		}
		
		public Status getStatus() {
			return status;
		}
		
		public void setStatus(Status status) {
			this.status = status;
		}	
	}	
}
