package de.uni_mannheim.app.periodic;

import java.util.Date;
import java.util.Timer;
import java.util.TimerTask;

import de.uni_mannheim.app.business.Status;
import de.uni_mannheim.app.controllers.AbstractController;
import de.uni_mannheim.app.network.commands.SendLocationCommand;
import de.uni_mannheim.aware.api.location.LocationData;
import de.uni_mannheim.aware.util.GeoUtil;

public class ArrivalTimeEstimationThread extends AbstractController {
	public static final int INTERVAL = 15000;
	private static ArrivalTimeEstimationThread instance;
	
	private Timer timer;
	
	private ArrivalTimeEstimationThread() {		
		this.timer = null;
	}
	
	public static ArrivalTimeEstimationThread getInstance() {
		if (instance == null)
			instance = new ArrivalTimeEstimationThread();
		
		return instance;
	}
	
	public synchronized void start() {
		timer = new Timer();
		
		timer.scheduleAtFixedRate(
			new TimerTask() { public void run() { runTimerTask(); } }, 
			0, INTERVAL);		
	}
	
	public synchronized void stop() {
		timer.cancel();
		timer = null;
	}
	
	private void runTimerTask() {
		try {
			if (workerContext.getLocation() == null) {
				System.out.println("[ArrivalTimeEstimationThread] Location unknown. Nothing to do."); return; }
			
			if (workerContext.getCurrentJob() == null) {
				System.out.println("[ArrivalTimeEstimationThread] Current job unknown. Nothing to do."); return; }
			
			if (!workerContext.getStatus().equals(Status.ASSIGNED)) {
				System.out.println("[ArrivalTimeEstimationThread] Status not assigned. Nothing to do."); return; }
			
			LocationData workerLocation = workerContext.getLocation();
			LocationData jobLocation = new LocationData(
				workerContext.getCurrentJob().getLatitide(), workerContext.getCurrentJob().getLongitude());
			
			System.out.println("[ArrivalTimeEstimationThread] Querying Google Directions API ...");		
			System.out.println("[ArrivalTimeEstimationThread] " +
					"From: " + workerLocation.getLatitude() + "/" + workerLocation.getLongitude() + " / " +
					"To: " + jobLocation.getLatitude() + "/" + jobLocation.getLongitude() );
		
			int minutes = GeoUtil.getTimeEstimate(workerLocation, jobLocation);
			
			workerContext.setEstimatedArrivalTime(new Date(System.currentTimeMillis()+minutes*60*1000));
			workerContext.setEstimatedArrivalTimeLastUpdated(new Date());
			
			System.out.println("[ArrivalTimeEstimationThread] Result: "+minutes+" minutes; Est. Arrival: "+workerContext.getEstimatedArrivalTime());
			
			ruleContext.getBusinessSensor().put("arrivalTimeEstimation", workerContext.getEstimatedArrivalTime());
		}
		catch (Exception e) {
			System.out.println("[ArrivalTimeEstimationThread] Error querying Google: "+e);		
			e.printStackTrace();
		}
	}
}
