package de.uni_mannheim.app.periodic;

import java.util.Timer;
import java.util.TimerTask;

import de.uni_mannheim.app.controllers.AbstractController;
import de.uni_mannheim.app.network.commands.GetWorkerContextCommand;
import de.uni_mannheim.app.network.commands.SendLocationCommand;

public class SendLocationThread extends AbstractController {	
	public static final int INTERVAL = 2000;
	private static SendLocationThread instance;
	
	private Timer timer;
	
	private SendLocationThread() {		
		this.timer = null;
	}
	
	public static SendLocationThread getInstance() {
		if (instance == null)
			instance = new SendLocationThread();
		
		return instance;
	}
	
	public synchronized void start() {
		timer = new Timer();
		
		timer.scheduleAtFixedRate(
			new TimerTask() { public void run() { runTimerTask(); } }, 
			INTERVAL, INTERVAL);		
	}
	
	public synchronized void stop() {
		timer.cancel();
		timer = null;
	}
	
	private void runTimerTask() {
		try {
			if (workerContext.getLocation() == null) {
				System.out.println("[SendLocationThread] No location known. Nothing to send");
				return;
			}
			
			System.out.println("[SendLocationThread] Sending location: "+workerContext.getLocation().getLatitude()+" / "+workerContext.getLocation().getLongitude()+"...");		
		
			net.send(
				new SendLocationCommand(workerContext.getWorkerId(), workerContext.getLocation()),					
				controllerContext.getLocationSensorController());
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}
}
