package de.uni_mannheim.aware.api;

import java.util.Enumeration;

public abstract class BooleanCondition extends Condition {
	private boolean conditionMet = false;

	public BooleanCondition(Sensor sensor) {
		super(sensor);
	}

	protected void setConditionMet(boolean conditionMet) {
		this.conditionMet = conditionMet;
	}
	
	public boolean isConditionMet() {
		return conditionMet;
	}

	protected void fireConditionChangedEvent() {
		synchronized (listeners) {
			for (Enumeration e = listeners.elements(); e.hasMoreElements(); )				
				((BooleanConditionListener) e.nextElement()).conditionChanged(this, conditionMet);
		}
	}	
	
	public void addConditionListener(BooleanConditionListener listener) {
		synchronized (listeners) {
			if (listeners.contains(listener))
				return;
			
			listeners.addElement(listener);			
		}
	}
	
	public boolean removeConditionListener(BooleanConditionListener listener) {
		synchronized (listeners) {
			return listeners.removeElement(listener);			
		}
	}


}

