package de.uni_mannheim.aware.api;

import java.util.Hashtable;

/**
 * Keeps the configuration of the APIs sensors. Exact keys for the 
 * individual sensors can be found in the concrete implementations.
 * 
 * <p>The syntax of the keys should have the form: <b>sensorname.propertyname</b><br />
 * For example: <b>time.interval</b>, or <b>bluetooth.timeout</b>
 *   
 * @author Philipp Heckel
 */
public class Config {
	private static Hashtable config = new Hashtable();
	
	public static void set(String key, String value) {
		config.put(key, value);
	}
	
	public static String get(String key) throws IllegalArgumentException {
		String obj = (String) config.get(key);
		
		if (obj == null)
			throw new IllegalArgumentException();
		
		return obj;
	}
	
	public static String get(String key, String defaultValue) {
		try {
			return get(key);
		}
		catch (IllegalArgumentException e) {
			return defaultValue;
		}
	}

	public static int getInt(String key) throws IllegalArgumentException {
		 return Integer.parseInt(get(key));
	}
	
	public static int getInt(String key, int defaultValue) {
		try {
			return getInt(key);
		}
		catch (IllegalArgumentException e) {
			return defaultValue;
		}
	}
}
