package de.uni_mannheim.aware.api;

import java.util.Calendar;

import de.uni_mannheim.aware.api.bluetooth.BluetoothCondition;
import de.uni_mannheim.aware.api.bluetooth.BluetoothData;
import de.uni_mannheim.aware.api.bluetooth.BluetoothDevice;
import de.uni_mannheim.aware.api.location.LocationCondition;
import de.uni_mannheim.aware.api.location.LocationData;
import de.uni_mannheim.aware.api.noise.NoiseLevelCondition;
import de.uni_mannheim.aware.api.noise.NoiseLevelData;
import de.uni_mannheim.aware.api.time.TimeCondition;
import de.uni_mannheim.aware.api.time.TimeData;
import de.uni_mannheim.aware.impl.UnsupportedPlatformException;
import de.uni_mannheim.aware.impl.SensorNotAvailableException;
import de.uni_mannheim.aware.impl.emu.EmulatedLocationSensor;

public class TestAPI {
	public static void main(String[] args) throws UnsupportedPlatformException, SensorNotAvailableException {	
		// Example: Rule matches if ...
		// * BT device aa:bb:cc:dd:ee is found
		// * and current time (seconds) is between :30 and :60, e.g. 15:04.38
		
		// Configure Sensors
		SensorFactory.getImpl().getTimeSensor().setInterval(1000);
		
		// Rules & Conditions
		Rule inCar = new Rule();
		
		BooleanCondition btCond = new BluetoothCondition() {			
			protected boolean matches(BluetoothData data) {
				return data.getDevices().contains(new BluetoothDevice("aa:bb:cc:dd:ee"));
			}
			
			public String toString() {
				return "{expr:\"BT device aa:bb:cc:dd:ee exists\",matches:"+isConditionMet()+"}";
			}
		};
		
		BooleanCondition inBetweenCond = new TimeCondition() {		
			protected boolean matches(TimeData data) {	
				int sec = data.getTime().get(Calendar.SECOND);								
				return sec >= 30 && sec <= 60;
			}
			
			public String toString() {
				return "{expr:\"30sec <= "+getTimeData().getTime().getTime()+" <= 60sec\",matches:"+isConditionMet()+"}";
			}
		};
		
		BooleanCondition inQuadratenCond = new LocationCondition() {		
			private LocationData officeLocation = new LocationData(49.5,8.5);
			
			protected boolean matches(LocationData currentLocation) {
				//System.out.println("distance: "+currentLocation.distance(paradeplatz));
				return currentLocation.distance(officeLocation) <= 500;
			}
			
			public String toString() {
				return "{expr:\"500m radius to "+getLocationData().getLatitude()+"/"+getLocationData().getLongitude()+", distance is "+getLocationData().distance(officeLocation)+"\",matches:"+isConditionMet()+"}";
			}			
		};
		/*
		BooleanCondition isLoudCond = null;
		try {
			isLoudCond = new NoiseLevelCondition() {
				
				protected boolean matches(NoiseLevelData currentNoiseLevelData) {
					return currentNoiseLevelData.getdB() > -10.0;
				}
				
				public String toString() {
					return "Current noise level is " + this.getNoiseLevelData().getdB() + " db";
				}
			};
		} catch (SensorNotAvailableException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}*/
		
	

		
		inCar.addCondition(btCond);
		inCar.addCondition(inBetweenCond);
		inCar.addCondition(inQuadratenCond);
		//inCar.addCondition(isLoudCond);
		
		inCar.setActive(true);
		
		RuleSet inCarSet = new RuleSet();
		inCarSet.addRule(inCar);
		
		// Okay. That's it. The rest is just to show it on the console!	
		inCar.addRuleListener(new RuleListener() {			
			public void ruleChanged(Rule rule, boolean matches) {	
				System.err.println("Rule changed: "+rule);
			}
		});
		
		btCond.addConditionListener(new BooleanConditionListener() {			
			public void conditionChanged(Condition condition, boolean matches) {
				System.err.println("BT Condition changed: "+condition);				
			}
		});
		
		inBetweenCond.addConditionListener(new BooleanConditionListener() {			
			public void conditionChanged(Condition condition, boolean matches) {
				System.err.println("Time Condition changed: "+condition);				
			}
		});		
		
		inQuadratenCond.addConditionListener(new BooleanConditionListener() {
			public void conditionChanged(Condition condition, boolean matches) {
				System.err.println("Quadrate Cond changed: "+condition);				
			}
		});
		
		/*isLoudCond.addConditionListener(new BooleanConditionListener() {
			public void conditionChanged(Condition condition, boolean matches) {
				System.err.println("Noise level changed: "+condition);				
			}
		});*/

		SensorFactory.getImpl().getLocationSensor().addListener(new SensorListener() {			
			public void sensorDataChanged(SensorData sensorData) {
				System.out.println("LocSensor changed: "+sensorData);				
			}
		});
		
		SensorFactory.getImpl().getBluetoothSensor().addListener(new SensorListener() {			
			public void sensorDataChanged(SensorData sensorData) {
				System.out.println("BT sensor changed: "+sensorData);				
			}
		});		
		
		SensorFactory.getImpl().getTimeSensor().addListener(new SensorListener() {			
			public void sensorDataChanged(SensorData sensorData) {
				System.out.println("Time sensor changed: "+sensorData);				
			}
		});				

		SensorFactory.getImpl().getNoiseLevelSensor().addListener(new SensorListener() {			
			public void sensorDataChanged(SensorData sensorData) {
				System.out.println("Noise level changed: "+sensorData);				
			}
		});				

	}
}
