package de.uni_mannheim.aware.api.battery;

import de.uni_mannheim.aware.api.BooleanCondition;
import de.uni_mannheim.aware.api.SensorData;
import de.uni_mannheim.aware.api.SensorFactory;
import de.uni_mannheim.aware.api.SensorListener;
import de.uni_mannheim.aware.impl.UnsupportedPlatformException;
import de.uni_mannheim.aware.impl.SensorNotAvailableException;

/**
 * 
 * @author Philipp Heckel
 */
public abstract class BatteryCondition extends BooleanCondition implements SensorListener  {
	public BatteryCondition() throws UnsupportedPlatformException, SensorNotAvailableException {
		super(SensorFactory.getImpl().getNoiseLevelSensor());
		setSensorData(new BatteryData());
	}
	
	/* @Override */
	public BatteryData getBatteryData() {
		return (BatteryData) getSensorData();
	}
	
	/* @Override */
	public void sensorDataChanged(SensorData sensorData) {
		if (! (sensorData instanceof BatteryData))
			return;
				
		// Update data
		BatteryData data = (BatteryData) sensorData;
		
		setSensorData(data);

		// Check changes
		boolean conditionMatches = matches(data);
		
		// Notify if changed
		if (isConditionMet() != conditionMatches) {
			setConditionMet(conditionMatches);	
			fireConditionChangedEvent();
		}		
	}
	
	/**
	 * 
	 * @param batteryData
	 * @return
	 */
	protected abstract boolean matches(BatteryData batteryData);	
}
