package de.uni_mannheim.aware.api.bluetooth;

import de.uni_mannheim.aware.api.BooleanCondition;
import de.uni_mannheim.aware.api.SensorData;
import de.uni_mannheim.aware.api.SensorFactory;
import de.uni_mannheim.aware.api.SensorListener;
import de.uni_mannheim.aware.impl.UnsupportedPlatformException;
import de.uni_mannheim.aware.impl.SensorNotAvailableException;

/**
 * 
 * @author Philipp Heckel
 */
public abstract class BluetoothCondition extends BooleanCondition implements SensorListener {
	public BluetoothCondition() throws UnsupportedPlatformException, SensorNotAvailableException {
		super(SensorFactory.getImpl().getBluetoothSensor());
		
		setSensorData(new BluetoothData());
	}
	
	public synchronized BluetoothData getBluetoothData() {
		return (BluetoothData) getSensorData();
	}
	
	/* @Override */
	public void sensorDataChanged(SensorData sensorData) {
		if (! (sensorData instanceof BluetoothData))
			return;
				
		// Update data
		BluetoothData data = (BluetoothData) sensorData;

		if (data.equals(getBluetoothData()))
			return;

		setSensorData(data);

		// Check changes
		boolean conditionMatches = matches(data);
		
		// Notify if changed
		if (isConditionMet() != conditionMatches) {
			setConditionMet(conditionMatches);	
			fireConditionChangedEvent();
		}		
	}
	
	protected abstract boolean matches(BluetoothData currentBluetoothData);	
}
