package de.uni_mannheim.aware.api.location;

import de.uni_mannheim.aware.api.BooleanCondition;
import de.uni_mannheim.aware.api.SensorData;
import de.uni_mannheim.aware.api.SensorFactory;
import de.uni_mannheim.aware.api.SensorListener;
import de.uni_mannheim.aware.impl.SensorNotAvailableException;
import de.uni_mannheim.aware.impl.UnsupportedPlatformException;

/**
 * 
 * @author Philipp Heckel
 */
public abstract class LocationCondition extends BooleanCondition implements SensorListener {
	public LocationCondition() throws UnsupportedPlatformException, SensorNotAvailableException {
		super(SensorFactory.getImpl().getLocationSensor());
		
		setSensorData(new LocationData());
	}
	
	public LocationData getLocationData() {
		return (LocationData) getSensorData();
	}
	
	/* @Override */
	public void sensorDataChanged(SensorData sensorData) {
		if (! (sensorData instanceof LocationData))
			return;
				
		// Update data
		LocationData data = (LocationData) sensorData.clone();

		if (data.equals(getLocationData()))
			return;

		setSensorData(data);

		// Check changes
		boolean conditionMatches = matches(data);
		
		// Notify if changed
		if (isConditionMet() != conditionMatches) {
			setConditionMet(conditionMatches);	
			fireConditionChangedEvent();
		}		
	}
	
	protected abstract boolean matches(LocationData currentLocation);	
}