package de.uni_mannheim.aware.api.wireless;

import de.uni_mannheim.aware.api.BooleanCondition;
import de.uni_mannheim.aware.api.SensorData;
import de.uni_mannheim.aware.api.SensorFactory;
import de.uni_mannheim.aware.api.SensorListener;
import de.uni_mannheim.aware.impl.SensorNotAvailableException;
import de.uni_mannheim.aware.impl.UnsupportedPlatformException;


public abstract class WirelessCondition extends BooleanCondition implements SensorListener {
	public WirelessCondition() throws UnsupportedPlatformException, SensorNotAvailableException {
		super(SensorFactory.getImpl().getWirelessSensor());
		
		setSensorData(new WirelessData());
	}
	
	public synchronized WirelessData getWirelessData() {
		return (WirelessData) getSensorData();
	}
	
	/* @Override */
	public void sensorDataChanged(SensorData sensorData) {
		if (! (sensorData instanceof WirelessData))
			return;
				
		// Update data
		WirelessData data = (WirelessData) sensorData;

		if (data.equals(getWirelessData()))
			return;

		setSensorData(data);

		// Check changes
		boolean conditionMatches = matches(data);
		
		// Notify if changed
		if (isConditionMet() != conditionMatches) {
			setConditionMet(conditionMatches);	
			fireConditionChangedEvent();
		}		
	}
	
	protected abstract boolean matches(WirelessData currentWirelessData);	
}
