package de.uni_mannheim.aware.impl;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Timer;
import java.util.TimerTask;
import javax.microedition.media.Manager;
import javax.microedition.media.MediaException;
import javax.microedition.media.Player;
import javax.microedition.media.control.RecordControl;

import de.uni_mannheim.aware.api.Config;
import de.uni_mannheim.aware.api.noise.NoiseLevelData;
import de.uni_mannheim.aware.api.noise.NoiseLevelSensor;
import de.uni_mannheim.aware.util.ByteConverter;
import de.uni_mannheim.aware.util.MathUtil;

/**
 * 
 * @author Lars Bakker
 */
public class CommonNoiseLevelSensor extends CommonSensor implements NoiseLevelSensor{
	/**
	 * Sleep time between the measurements (in ms).
	 * Can be overridden with {@link Config#set(String, String)}
	 * using the key <b>noise.sleep</b>.
	 * 
	 * <p><b>Note</b>: Can be changed during runtime!
	 */
	private static final int DEFAULT_SLEEP_TIME = 1500;
	
	/**
	 * Time to record the audio (in ms).
	 * Can be overridden with {@link Config#set(String, String)}
	 * using the key <b>noise.recordtime</b>.
	 * 
	 * <p><b>Note</b>: Can be changed during runtime!
	 */
	private static final int DEFAULT_RECORD_TIME = 1500;
	
	private static NoiseLevelSensor instance;
		
	private NoiseLevelData data;
	private Player p;
	
	
	private Timer timer;
	private TimerTask task;
	
	private CommonNoiseLevelSensor() throws SensorNotAvailableException {
		this.data = new NoiseLevelData();
		String encoding;
		try{
	    	p = Manager.createPlayer("capture://audio?encoding=pcm");
	        p.realize();
	        encoding = p.getContentType();
	        p.close();
		}catch(Exception e){
        	throw new SensorNotAvailableException();
		}
        if (!(encoding.equalsIgnoreCase("audio/x-wav") || encoding.equalsIgnoreCase("audio/wav"))){
        	throw new SensorNotAvailableException();
        }
	}
		
	/**
	 * Singleton!
	 */
	public static NoiseLevelSensor getInstance() throws SensorNotAvailableException {
		if (instance != null)
			return instance;
		
		instance = new CommonNoiseLevelSensor();
		return instance;
	}	
	
	
	/* @Override */
	public synchronized void start() {
		if (this.getListeners().size() > 0){
			timer = new Timer();
			task = new NoiseLevelTask();
			timer.schedule(task,0, Config.getInt("noise.sleep", DEFAULT_SLEEP_TIME));
		}
	}
	
	/* @Override */
	public synchronized void stop() {
		if (timer != null){
			timer.cancel();
			task.cancel();
			timer = null;
		}
	}



	public NoiseLevelData getNoiseLevel() {
		return data;
	}


	
	class NoiseLevelTask extends TimerTask {
		private Thread thread = null;
		
		public boolean cancel() {
			thread.interrupt();
			return super.cancel();
		}
				
		public void run() {
			thread = Thread.currentThread();
            try{ 
		    	p = Manager.createPlayer("capture://audio?encoding=pcm");
		        p.realize();
	            RecordControl rc = (RecordControl)p.getControl("RecordControl");
	            ByteArrayOutputStream output = new ByteArrayOutputStream();
	            
	            rc.setRecordStream(output); 
	
	            rc.startRecord();
	            p.start();
	            System.out.println("Start sleeping");
				Thread.sleep(Config.getInt("noise.recordtime", DEFAULT_RECORD_TIME));
				System.out.println("Stop sleeping");
	            rc.commit();               
	            //messageItem.setText("done!");
	            byte[] rec = output.toByteArray(); 
	           
	            int sampleCount = ((rec.length - 44) >> 1);	 
	            int offset = 44;
		
	            long squarelong = 0;
	
	            for (int i = (int)offset; i < rec.length; i += 2){
	        	    short sample = ByteConverter.readShortBigEndian(rec, i);
	        	    sample = (short) Math.abs((int)sample);
	        	    squarelong += sample*sample;
	            }
	            double squares = (double)squarelong;
	            
	            squares = squares /(double)sampleCount;
	            
	            squares = Math.sqrt(squares);
	                       
	            // 32768 = max value of the amplitude
	            
	            double dbavg = 20 * MathUtil.log10(squares / 32768.0);
	            NoiseLevelData currentLvl = new NoiseLevelData(dbavg);
	            if (!currentLvl.equals(data)){
		            data = currentLvl;	          
	            	System.out.println("NoiseLevelData: Sending Event");
		            fireSensorDataChangedEvent(currentLvl);	            	
	            }else{
	            	System.out.println("NoiseLevelData: NOT Sending because nothing has changed");
	            }
	            
            } catch (IOException ioe) {
	        	// We ignore this
	        } catch (MediaException me) {
	        	// This should not happen
	        } catch (InterruptedException ie) {
	        	System.out.println("Got interrupted. Stopping recording immediatly ");
	        } catch (Exception e){
	        	// also ignore this
	        } finally {
	        	p.close();
	        }
		}	
	}
/*
	public int getRecordMillis() {
		return recordMillis;
	}

	public void setRecordMillis(int recordMillis) {
		this.recordMillis = recordMillis;
	}

	public int getSleepMillis() {
		return sleepMillis;
	}

	public void setSleepMillis(int sleepMillis) {
		this.sleepMillis = sleepMillis;
	}*/
}
