package de.uni_mannheim.aware.impl;

import java.util.Enumeration;
import java.util.Vector;

import de.uni_mannheim.aware.api.Sensor;
import de.uni_mannheim.aware.api.SensorData;
import de.uni_mannheim.aware.api.SensorFactory;
import de.uni_mannheim.aware.api.SensorListener;

/**
 * The common sensor implements simple management methods such
 * as listener handling, or the availability status. Every concrete
 * sensors should extend this class.
 *  
 * @author Philipp Heckel
 */
public abstract class CommonSensor implements Sensor {
	private Vector listeners;
	private boolean available;

	public CommonSensor() {
		listeners = new Vector();
		available = false;
	}
	 
	/* @Override */
	public synchronized void addListener(SensorListener listener) {
		listeners.addElement(listener);
		
		if (listeners.size() == 1)
			start();
	}

	/* @Override */
	public synchronized void removeListener(SensorListener listener) {
		listeners.removeElement(listener);
		
		if (listeners.size() == 0)
			stop();
	}
	
	/**
	 * Fires an event to the sensor listeners. This method should be called
	 * by the concrete sensors if the sensor data has changed.
	 * 
	 * @param sensorData Updated sensor data, should be a subclass of {@link SensorData}
	 */
	protected void fireSensorDataChangedEvent(SensorData sensorData) {
		synchronized (listeners) {
			for (Enumeration e = listeners.elements(); e.hasMoreElements(); ) {
				SensorListener l = ((SensorListener) e.nextElement());
				System.out.println("notifying "+l);
				l.sensorDataChanged(sensorData);
			}
		}		
	}	
	
	/**
	 * Method called when the first listener registers to the
	 * sensor. 
	 */
	protected void start() {
		
	}
	
	/**
	 * Method called when the last listener is removed from the 
	 * sensor.
	 */
	protected void stop() {
		
	}

	protected Vector getListeners() {
		return listeners;
	}
}

