package de.uni_mannheim.aware.impl;

import java.util.Calendar;
import java.util.Enumeration;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Vector;

import de.uni_mannheim.aware.api.Config;
import de.uni_mannheim.aware.api.time.TimeData;
import de.uni_mannheim.aware.api.time.TimeSensor;
import de.uni_mannheim.aware.impl.emu.EmulatedBluetoothSensor;
import de.uni_mannheim.aware.impl.nokia.NokiaWirelessSensor;

/**
 * <p><b>Configuration parameters</b>:
 * <ul>
 *   <li><b>time.interval</b>: interval to which the update event is fired (in ms).
 * </ul>
 * 
 * @author Philipp Heckel
 */
public class CommonTimeSensor extends CommonSensor implements TimeSensor {
	/**
	 * Interval to which the update event is fired.
	 * Can be overridden with {@link Config#set(String, String)}
	 * using the key <b>time.interval</b>.
	 */
	public static final int DEFAULT_INTERVAL = 1000;
	
	private static CommonTimeSensor instance;
	
	private Timer timer;
	private TimeData timeData;
	
	private int interval;
	
	/**
	 * Singleton!
	 */
	private CommonTimeSensor() {
		this.timer = null;
		this.timeData = new TimeData(Calendar.getInstance());
		
		this.interval = Config.getInt("time.interval", DEFAULT_INTERVAL);
	}
	
	public static TimeSensor getInstance() {
		if (instance != null)
			return instance;
		
		instance = new CommonTimeSensor();
		return instance;
	}
	
	/* @Override */
	public void setInterval(int interval) {
		this.interval = interval;
	}

	/* @Override */
	public int getInterval() {
		return interval;
	}
	
	/* @Override */
	public synchronized void start() {		
		timer = new Timer();
		timer.schedule(
			new TimerTask() { public void run() { timerTaskEvent(); } },
			getInterval(), getInterval() );		
	}
	
	/* @Override */
	public synchronized void stop() {
		timer.cancel();
		timer = null;
	}
	
	/* @Override */
	public boolean isAvailable() {
		return true;
	}
	
	/* @Override */
	public TimeData getTimeData() {
		return timeData;
	}

	private void timerTaskEvent() {
		synchronized (timeData) {
			timeData = new TimeData(Calendar.getInstance());	
		}
		
		fireSensorDataChangedEvent(timeData);
	}
}
