package de.uni_mannheim.aware.impl.emu;

import java.util.Random;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Vector;

import de.uni_mannheim.aware.api.SensorData;
import de.uni_mannheim.aware.api.bluetooth.BluetoothData;
import de.uni_mannheim.aware.api.bluetooth.BluetoothDevice;
import de.uni_mannheim.aware.api.bluetooth.BluetoothSensor;
import de.uni_mannheim.aware.api.location.LocationData;
import de.uni_mannheim.aware.impl.CommonSensor;
import de.uni_mannheim.aware.impl.SensorNotAvailableException;
import de.uni_mannheim.aware.util.VectorUtil;

public class EmulatedBluetoothSensor extends CommonSensor implements BluetoothSensor {
	private static EmulatedBluetoothSensor instance;
	public static int INTERVAL = 10000;
	
	private static final BluetoothDevice[] demoDevices = new BluetoothDevice[] {
		new BluetoothDevice("aabbccddee"),
		new BluetoothDevice("0011223344"),
		/*new BluetoothDevice("9988776655"),
		new BluetoothDevice("ffeeddccbb"),
		new BluetoothDevice("0123456789"),*/
		new BluetoothDevice("002556D0148C")
	};

	private int currentDeviceIndex;
	private Timer timer;
	private TimerTask timerTask;
	private BluetoothData data;	                                                                         
	
	private EmulatedBluetoothSensor() {
		this.currentDeviceIndex = 0;
		this.timer = new Timer();
		this.data = new BluetoothData();
		
		// Start timer (= emulated BT discovery!)
		this.timerTask = new TimerTask() {			
			public void run() {
				timerTaskEvent();
			}
		};		
	}
	
	/**
	 * Singleton!
	 * @return
	 */
	public static BluetoothSensor getInstance() throws SensorNotAvailableException {
		if (instance != null)
			return instance;
		
		instance = new EmulatedBluetoothSensor();
		return instance;
	}	
	
	/* @Override */
	public void start() {		
		timer.schedule(timerTask, INTERVAL, INTERVAL);		
	}
	
	/* @Override */
	public void stop() {
		timer.cancel();
	}
	
	/* @Override */
	public boolean isAvailable() {
		return true;
	}	
	
	/* @Override */	
	public synchronized BluetoothData getBluetoothData() {
		return data;
	}
	
	/* @Override */	
	public synchronized SensorData getSensorData() {
		return getBluetoothData();
	}	
	
	private void timerTaskEvent() {
		// Clone existing object
		BluetoothData newData;
		
		synchronized (data) {
			newData = (BluetoothData) data.clone();
		}
		
		// randomly choose devices and add/remove them
		currentDeviceIndex = (currentDeviceIndex + 1) % demoDevices.length;
		int vanishDeviceIndex = (currentDeviceIndex + 1) % demoDevices.length;
		
		BluetoothDevice discoveredDevice = demoDevices[currentDeviceIndex];//new Random(System.currentTimeMillis()).nextInt(demoDevices.length)];
		BluetoothDevice vanishedDevice = demoDevices[vanishDeviceIndex];//new Random(System.currentTimeMillis()+12345).nextInt(demoDevices.length)];
			
		if (!newData.getDevices().contains(discoveredDevice)) {
			newData.getDevices().addElement(discoveredDevice);							
			newData.setDiscovered(VectorUtil.fromArray(new BluetoothDevice[] { discoveredDevice }));
		}
		
		if (newData.getDevices().contains(vanishedDevice)) {
			newData.getDevices().removeElement(vanishedDevice);
			newData.setVanished(VectorUtil.fromArray(new BluetoothDevice[] { vanishedDevice }));						
		}
		
		// Issue event if changed
		synchronized (data) {
			if (newData.equals(data)) 
				return; 
			
			//System.out.println("BT Sensor changed: "+data);
			
			data = newData;
			fireSensorDataChangedEvent(data);
		}		
		
		
	}

}
