package de.uni_mannheim.aware.impl.emu;

import java.util.Random;
import java.util.Timer;
import java.util.TimerTask;

import de.uni_mannheim.aware.api.SensorData;
import de.uni_mannheim.aware.api.bluetooth.BluetoothSensor;
import de.uni_mannheim.aware.api.location.LocationData;
import de.uni_mannheim.aware.api.location.LocationSensor;
import de.uni_mannheim.aware.impl.CommonSensor;

public class EmulatedLocationSensor extends CommonSensor implements LocationSensor {
	private static LocationSensor instance;

	private static final LocationData startData = new LocationData(49.5,8.5,2000);
	private static final LocationData upperBoundary = new LocationData(50, 9);
	private static final LocationData lowerBoundary = new LocationData(49, 8);
	private static final double movementPerStep = 0.001; 
	
	private Timer timer;
	private LocationData data;	     
	
	private EmulatedLocationSensor() {
		this.timer = null;
		this.data = startData;
	}
	
	/**
	 * Singleton!
	 * @return
	 */
	public static LocationSensor getInstance() {
		if (instance != null)
			return instance;
		
		instance = new EmulatedLocationSensor();
		return instance;
	}		
	
	/* @Override */
	public synchronized LocationData getLocation() {
		return data;
	}
	
	/* @Override */
	public synchronized SensorData getSensorData() {
		return getLocation();
	}		
	
	/* @Override */
	public synchronized void start() {
		// Cancel the old one
		if (timer != null)
			return;
		
		// Start timer (= emulated BT discovery!)
		timer = new Timer();
		
		timer.schedule(
			new TimerTask() { public void run() { timerTaskEvent(); } },
			5000, 5000);
	}
	
	/* @Override */
	public synchronized void stop() {
		if (timer != null)
			timer.cancel();
	}
	
	/* @Override */
	public boolean isAvailable() {
		return true;
	}
	
	private void timerTaskEvent() {		
		// Clone existing object
		LocationData newData;
		
		synchronized (data) {
			newData = (LocationData) data.clone();
		}
		
		// Randomly move by 0.1 or -0.1
		double latMove = (new Random(System.currentTimeMillis()).nextInt(2) == 1) ? movementPerStep : -movementPerStep;
		double lngMove = (new Random(System.currentTimeMillis()+1234).nextInt(2) == 1) ? movementPerStep : -movementPerStep;		
		
		newData.setLatitude(newData.getLatitude()+latMove);
		newData.setLongitude(newData.getLongitude()+lngMove);
		
		// Check boundaries for latitude
		if (newData.getLatitude() < lowerBoundary.getLatitude())
			newData.setLatitude(lowerBoundary.getLatitude());
		
		else if (newData.getLatitude() > upperBoundary.getLatitude())
			newData.setLatitude(upperBoundary.getLatitude());
		System.out.println("new location4");
		// Check boundaries for longitude
		if (newData.getLongitude() < lowerBoundary.getLongitude())
			newData.setLongitude(lowerBoundary.getLongitude());
		
		else if (data.getLongitude() > upperBoundary.getLongitude())
			newData.setLongitude(upperBoundary.getLongitude());
		
		// Issue event if changed
		if (newData.equals(data)) 
			return; 
		
		//System.out.println("Loc Sensor changed: "+data);
		data = newData;
		fireSensorDataChangedEvent(data);
	}
}
