package de.uni_mannheim.aware.impl.emu;

import java.util.Random;
import java.util.Timer;
import java.util.TimerTask;

import de.uni_mannheim.aware.api.SensorData;
import de.uni_mannheim.aware.api.SensorFactory;
import de.uni_mannheim.aware.api.wireless.WirelessAccessPoint;
import de.uni_mannheim.aware.api.wireless.WirelessData;
import de.uni_mannheim.aware.api.wireless.WirelessSensor;
import de.uni_mannheim.aware.impl.CommonSensor;
import de.uni_mannheim.aware.impl.SensorNotAvailableException;
import de.uni_mannheim.aware.util.VectorUtil;

public class EmulatedWirelessSensor extends CommonSensor implements WirelessSensor {
	private static WirelessSensor instance;
	
	// Should lead to: 49.4913896, 8.4665921
	private static final WirelessAccessPoint[] demoDevices = new WirelessAccessPoint[] {
		new WirelessAccessPoint(new byte[] { 0x00, 0x21, 0x29, (byte) 0x83, 0x61, 0x1E}, "Lalala", -65),
		new WirelessAccessPoint(new byte[] { 0x00, 0x23, 0x08, 0x50, (byte) 0xD3, 0x21}, "EasyBox-Soner", -69),
		new WirelessAccessPoint(new byte[] { 0x00, 0x19, (byte) 0xCB, (byte) 0x86, 0x7E, 0x2E}, "PhiLeMiKa", -42),
		new WirelessAccessPoint(new byte[] { 0x00, 013, 0x49, (byte) 0xf4, (byte) 0x84, 0x62}, "xakac"),
		new WirelessAccessPoint(new byte[] { 0x00, 0x1f, 0x3f, (byte) 0xd0, (byte) 0xc1, 0x49}, "FRITZ!Box Fon WLAN 7270"),
		new WirelessAccessPoint(new byte[] { 0x00, 0x23, 0x08, 0x70, (byte) 0x83, 0x41}, "EasyBox-708331"),
		new WirelessAccessPoint(new byte[] { 0x00, 0x1c, 0x28, (byte) 0xae, 0x4e, 0x27}, "ALICE-WLAN58"),
		new WirelessAccessPoint(new byte[] { 0x00, 0x23, 0x08, (byte) 0xfa, (byte) 0xd5, 0x78}, "EasyBox-FAD558"),
		new WirelessAccessPoint(new byte[] { 0x00, 0x24, (byte) 0xb2, (byte) 0x8c, (byte) 0xce, 0x74}, "WLAN_AFE"),
	};

	private Timer timer;
	private TimerTask timerTask;
	private WirelessData data;	                                                                         
	
	private EmulatedWirelessSensor() {
		this.timer = new Timer();
		this.data = new WirelessData();
		
		// Start timer (= emulated BT discovery!)
		this.timerTask = new TimerTask() {			
			public void run() {
				timerTaskEvent();
			}
		};		
	}
	
	/**
	 * Singleton!
	 * @return
	 */
	public static WirelessSensor getInstance() throws SensorNotAvailableException {
		if (instance != null)
			return instance;
		
		instance = new EmulatedWirelessSensor();
		return instance;
	}	
	
	/* @Override */
	public void start() {		
		timer.schedule(timerTask, 1000, 1000);		
	}
	
	/* @Override */
	public void stop() {
		timer.cancel();
	}
	
	/* @Override */	
	public WirelessData getWirelessDevices() {
		return data;
	}
	
	/* @Override */
	public SensorData getSensorData() {
		return getWirelessDevices();
	}		
	
	private void timerTaskEvent() {		
		// Clone existing object
		WirelessData newData;
		
		synchronized (data) {
			newData = (WirelessData) data.clone();
		}
		
		// randomly choose devices and add/remove them
		WirelessAccessPoint discoveredDevice = demoDevices[new Random(System.currentTimeMillis()).nextInt(demoDevices.length)];
		WirelessAccessPoint vanishedDevice = demoDevices[new Random(System.currentTimeMillis()+12345).nextInt(demoDevices.length)];
			
		if (!newData.getDevices().contains(discoveredDevice)) {
			newData.getDevices().addElement(discoveredDevice);							
			newData.setDiscovered(VectorUtil.fromArray(new WirelessAccessPoint[] { discoveredDevice }));
		}
		
		if (newData.getDevices().contains(vanishedDevice)) {
			newData.getDevices().removeElement(vanishedDevice);
			newData.setVanished(VectorUtil.fromArray(new WirelessAccessPoint[] { vanishedDevice }));						
		}
		
		// Issue event if changed
		synchronized (data) {
			if (newData.equals(data)) 
				return; 
			
			//System.out.println("BT Sensor changed: "+data);
			
			data = newData;
			fireSensorDataChangedEvent(data);
		}		
		
		
	}


}
