package de.uni_mannheim.aware.impl.nokia;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Timer;
import java.util.TimerTask;

import javax.microedition.io.Connector;
import javax.microedition.io.StreamConnection;

import de.uni_mannheim.aware.api.SensorData;
import de.uni_mannheim.aware.api.battery.BatteryData;
import de.uni_mannheim.aware.api.battery.BatterySensor;
import de.uni_mannheim.aware.impl.CommonSensor;
import de.uni_mannheim.aware.impl.SensorNotAvailableException;

public class NokiaBatterySensor extends CommonSensor implements BatterySensor {
	/**
	 * If the sensor is enabled, it polls the native sensor regularly.
	 * This value defines the milliseconds to wait between the cycles
	 */
	public static final int SCAN_SLEEP_TIME = 10106;
	
	private static BatterySensor instance;
	
	private BatteryData data;
	
	private Timer timer;	
	
	private NokiaBatterySensor() {
		data = new BatteryData();
	}
	
	/**
	 * Singleton!
	 * @throws SensorNotAvailableException 
	 */
	public static BatterySensor getInstance() throws SensorNotAvailableException {
		if (instance != null)
			return instance;
		
		instance = new NokiaBatterySensor();
		return instance;
	}

	
	/* @Override */
	public synchronized void start() {
		if (timer != null)
			return;
		
		timer = new Timer();
		//timer.schedule(new DiscoveryTimerTask(), 0); // schedule for one (!) time
		timer.schedule(new DiscoveryTimerTask(), SCAN_SLEEP_TIME, SCAN_SLEEP_TIME); // periodically
	}
	
	/* @Override */
	public synchronized void stop() {
		if (timer == null)
			return;
		
		timer.cancel();
		timer = null;
	}	
	
	/* @Override */
	public synchronized BatteryData getBatteryData() {
		return data;
	}		
	
	/* @Override */
	public synchronized SensorData getSensorData() {
		return getBatteryData();
	}			
	
	private synchronized void setBatteryData(BatteryData data) {
		this.data = data;
	}
	
	private class DiscoveryTimerTask extends TimerTask {
		public void run() {		
			BatteryData newData;
			
			try {						
				// Discover synchronously! Blocking!	
				 newData = retrieveStatus();							
			}
			catch (Exception e) {
				newData = new BatteryData();
			}
			
			// Fire if new data available!
			if (!getBatteryData().equals(newData)) {
				setBatteryData(newData);
				fireSensorDataChangedEvent(newData);
			}				
		}
				
		public synchronized BatteryData retrieveStatus() throws InterruptedException, IOException {
			// Open internal connection
            StreamConnection conn = (StreamConnection)Connector.open("socket://127.0.0.1:8100");
            OutputStream out = conn.openOutputStream();
            
            // Send request
            byte[] request = "b".getBytes();
            out.write(request, 0, request.length);
            out.flush();
            out.close();
            
            // Read response           
            InputStream in = conn.openInputStream();
            
            int chargeLevel = in.read();
            byte chargerStatus = (byte) in.read();
            
            in.close();
            conn.close();

			return new BatteryData(chargerStatus, chargeLevel);
		}
	}	
}
