package de.uni_mannheim.aware.impl.nokia;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Date;
import java.util.Enumeration;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Vector;

import javax.microedition.io.Connector;
import javax.microedition.io.StreamConnection;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Form;

import de.uni_mannheim.aware.api.SensorData;
import de.uni_mannheim.aware.api.SensorListener;
import de.uni_mannheim.aware.api.bluetooth.BluetoothData;
import de.uni_mannheim.aware.api.bluetooth.BluetoothDevice;
import de.uni_mannheim.aware.api.bluetooth.BluetoothSensor;
import de.uni_mannheim.aware.api.wireless.WirelessAccessPoint;
import de.uni_mannheim.aware.api.wireless.WirelessData;
import de.uni_mannheim.aware.api.wireless.WirelessSensor;
import de.uni_mannheim.aware.impl.CommonBluetoothSensor;
import de.uni_mannheim.aware.impl.CommonSensor;
import de.uni_mannheim.aware.impl.SensorNotAvailableException;
import de.uni_mannheim.aware.util.StringUtil;
import de.uni_mannheim.aware.util.TimedList;
import de.uni_mannheim.aware.util.VectorUtil;
import de.uni_mannheim.aware.util.TimedList.ExpiryListener;

public class NokiaWirelessSensor extends CommonSensor implements WirelessSensor, ExpiryListener {
	/**
	 * If the sensor is enabled, it polls the native sensor regularly.
	 * This value defines the milliseconds to wait between the cycles
	 */
	public static final int SCAN_SLEEP_TIME = 3000;
	
	public static final int DEVICE_TIMEOUT = 60000;
	
	private static WirelessSensor instance;

	private TimedList currentDeviceList;
	
	private Timer timer;

	private NokiaWirelessSensor() {
		currentDeviceList = new TimedList();
		currentDeviceList.addExpiryListener(this);
		
		timer = null;
	}
	
	/**
	 * Singleton!
	 * @throws SensorNotAvailableException 
	 */
	public static WirelessSensor getInstance() throws SensorNotAvailableException {
		if (instance != null)
			return instance;
		
		instance = new NokiaWirelessSensor();
		return instance;
	}		
	
	/* @Override */
	public synchronized void start() {
		if (timer != null)
			return;
		
		timer = new Timer();
		//timer.schedule(new DiscoveryTimerTask(), 0); // schedule for one (!) time
		timer.schedule(new DiscoveryTimerTask(), SCAN_SLEEP_TIME, SCAN_SLEEP_TIME); // periodically
	}
	
	/* @Override */
	public synchronized void stop() {
		if (timer == null)
			return;
		
		timer.cancel();
		timer = null;
	}
	
	/* @Override */
	public void objectsExpired(Vector expiredObjects) {
		// Make wifi data object (current devices / vanished devices)
		WirelessData data = new WirelessData();
		
		synchronized (currentDeviceList) {
			data.setDevices(currentDeviceList.getObjects());
			data.setVanished(expiredObjects);

			/*for (Enumeration e = expiredObjects.elements(); e.hasMoreElements(); ) {
				synchronized (TestMidlet3.instance) {
					Form a = (Form) Display.getDisplay(TestMidlet3.instance).getCurrent();
					a.append("expired: "+((WirelessAccessPoint) e.nextElement()).getSsid()+"\n");					
				}
			}	*/
		}
		
		// Fire!
		fireSensorDataChangedEvent(data);		
	}		
	
	/** 
	 * Called by the {@link DiscoveryTimerTask} if new devices have been
	 * discovered.
	 * 
	 * @param addedObjects
	 */
	public synchronized void objectsAdded(Vector addedObjects) {
		// Make wifi data object (current devices / new devices)
		WirelessData data = new WirelessData();
		
		data.setDevices(currentDeviceList.getObjects());
		data.setDiscovered(addedObjects);

		// Fire!
		fireSensorDataChangedEvent(data);	
	}	
	
	/* @Override */
	public synchronized WirelessData getWirelessDevices() {
		WirelessData data = new WirelessData();		
		data.setDevices(currentDeviceList.getObjects());
		
		return data;
	}	
	
	/* @Override */
	public synchronized SensorData getSensorData() {
		return getWirelessDevices();
	}			
	
	private class DiscoveryTimerTask extends TimerTask {
		private Vector newDeviceList;
		
		public void run() {			
			try {						
				// Discover synchronously! Blocking!					
				discoverSynchronously();
				
				// Fire notifications for new devices
				synchronized (newDeviceList) {
					if (newDeviceList.size() > 0)
						objectsAdded(newDeviceList);		
				}			
			}
			catch (NullPointerException e) {
				// Whatever 
				/*synchronized (TestMidlet3.instance) {
				       Form a = (Form) Display.getDisplay(TestMidlet3.instance).getCurrent();
						a.append("\n\n"+e.toString());
				}*/
			}
			catch (IOException e) {
				// Whatever
				/*synchronized (TestMidlet3.instance) {				
	            Form a = (Form) Display.getDisplay(TestMidlet3.instance).getCurrent();
				a.append("\n\n"+e.toString());			
				}*/
			}
			catch (Exception e) {
				/*synchronized (TestMidlet3.instance) {
	            Form a = (Form) Display.getDisplay(TestMidlet3.instance).getCurrent();
				a.append("\n\n"+e.toString());
				}*/
			}
		}
				
		public void discoverSynchronously() throws IOException {
			newDeviceList = new Vector();
            /*synchronized (TestMidlet3.instance) {
				Form a = (Form) Display.getDisplay(TestMidlet3.instance).getCurrent();
				a.append("\ndiscoverSynchronously\n");	
			}*/
            
			// Open internal connection
            StreamConnection conn = (StreamConnection)
            	Connector.open("socket://127.0.0.1:8100");
            
            // Send request
            OutputStream out = conn.openOutputStream();

            byte[] request = "w".getBytes();
            out.write(request, 0, request.length);
            out.flush();
            out.close();
            
            // Read response           
            InputStream in = conn.openInputStream();
            
            int wifiCount = in.read();
            
            for (int i = 0; i<wifiCount; i++) {
            	// Read from socket: Protocol !
            	byte[] bssid = new byte[6];
            	byte[] ssid = new byte[32];
            	int rxLevel = 0;
            	
            	if (in.read(bssid, 0, bssid.length) < bssid.length)
            		break;
            	
            	if (in.read(ssid, 0, ssid.length) < ssid.length)
            		break;
            	
            	if ((rxLevel = in.read()) < 0)
            		break;
            	
            	// Create AP
    			WirelessAccessPoint device = new WirelessAccessPoint(bssid, StringUtil.getCString(ssid), rxLevel);
    			    			
    	        /*synchronized (TestMidlet3.instance) {
    				Form a = (Form) Display.getDisplay(TestMidlet3.instance).getCurrent();
    				a.append(" "+device.getSsid()+", ");
    			}   */ 			
    			
    			// Add or update lists
    			synchronized (currentDeviceList) {
    				if (!currentDeviceList.contains(device))
    					newDeviceList.addElement(device);				

    				// Update existing, or add new entry 
    				currentDeviceList.replace(device, DEVICE_TIMEOUT);
    			}            	
            }
                       
            //Form a = (Form) Display.getDisplay(TestMidlet3.instance).getCurrent();
			//a.append("\n\n"+currentDeviceList.dump());
			
            try {
            	in.close();
            	conn.close();
                /*synchronized (TestMidlet3.instance) {
    				Form a = (Form) Display.getDisplay(TestMidlet3.instance).getCurrent();
    				a.append(" close discover\n ");
    			}    */	
            }
            catch (Exception e) {
            	// Nothing.
            }
		}
	}
}
