package de.uni_mannheim.aware.network;



	

import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;

import javax.microedition.io.Connector;
import javax.microedition.io.HttpConnection;
import javax.microedition.location.Coordinates;

import org.json.me.JSONArray;
import org.json.me.JSONException;
import org.json.me.JSONObject;

import de.uni_mannheim.aware.util.StringUtil;
import de.uni_mannheim.aware.util.VectorUtil;

/**
 * @author Moritz Wchter
 *
 */
public class Network { 
	/**
	 * The host of the web page that is requesting the location.
	 * This field is required by the protocol.
	 * 
	 * @see http://code.google.com/apis/gears/geolocation_network_protocol.html#request
	 */
	private static final String REQUEST_HOST = "uni-mannheim.de";
			
	/**
	 * URL of server
	 * 
	 */
	private static String url = "http://vsrv28.informatik.uni-mannheim.de:8080/AwareServer-war/resources/contextapi/";
	

	public boolean test() throws IOException {
		boolean rule = false;
		String sensorString = "bluetooth";	
		int ruleid = 1;		
		return updateSensor(1,"bluetooth", "5 devices in reach");
	}
	
	public class RuleSender extends Thread{
		private int ruleid;
		private boolean value;
		private String name;
		public RuleSender(int ruleid, boolean value, String name){
			this.ruleid = ruleid;
			this.value = value;
			this.name = name;
		}
		public void run() {

				String uri =  url + "rules/" + ruleid;
				
				String valueString;
				if(value)
					valueString = "true";
				else
					valueString = "false";
				
				JSONObject ruleInJSON  = new JSONObject();
				
				try {
					ruleInJSON.put("ruleSatisfied", valueString);
					ruleInJSON.put("ruleName", name);
					}
				catch (JSONException e) {
					// Nothing. Error cannot happen!
				}		
				try {
					putData(uri,ruleInJSON);
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
		}		
	}

	public class SensorSender extends Thread{
		int sensorId;
		String sensorStr;
		String details;
		public SensorSender(int sensorId, String sensorStr, String details){
			this.sensorStr = sensorStr;
			this.details = details;
			this.sensorId = sensorId;
		}
		public void run() {

				String uri = url + "sensors/" + sensorId;
				
				
				JSONObject ruleInJSON  = new JSONObject();
				
				try {
					ruleInJSON.put("name", sensorStr);
					ruleInJSON.put("data", details);
					}
				catch (JSONException e) {
					// Nothing. Error cannot happen!
				}		
				try {
					putData(uri,ruleInJSON);
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
		}		
	}
	
	
	public class ConditionSender extends Thread{
		private int ruleId;
		private int conditionId;
		private boolean value;
		private String name, details;
		public ConditionSender(int ruleId, int conditionId, boolean value, String name, String details){
			this.ruleId = ruleId;
			this.value = value;
			this.name = name;
			this.details = details;
			this.conditionId = conditionId;
		}
		public void run() {

			    String uri =  url + "conditions/" + ruleId +"/"+ conditionId;				
				String valueString;
				if(value)
					valueString = "true";
				else
					valueString = "false";
				
				JSONObject ruleInJSON  = new JSONObject();
				
				try {
					ruleInJSON.put("conditionMet", valueString);
					ruleInJSON.put("details", details);
					ruleInJSON.put("name", name);
					}
				catch (JSONException e) {
					// Nothing. Error cannot happen!
				}		
				try {
					putData(uri,ruleInJSON);
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
		}		
	}
	
	
	public static synchronized boolean updateRule(int ruleid, boolean value, String name){
		Network network = new Network();
		RuleSender ruleSender = network.new RuleSender(ruleid, value, name);
		ruleSender.start();
		return true;
	}

	public static synchronized boolean updateCondition(int ruleId, int conditionId, boolean value, String name, String details){
		Network network = new Network();
		ConditionSender conditionSender = network.new ConditionSender(ruleId, conditionId, value, name, details);
		conditionSender.start();
		return true;		
	}

	public static synchronized boolean updateSensor(int sensorId, String sensorStr, String details){
		Network network = new Network();
		SensorSender sensorSender = network.new SensorSender(sensorId, sensorStr, details);
		sensorSender.start();
		return true;		
	}

	
	/**
	 * 
	 * @see http://www.devx.com/getHelpOn/10MinuteSolution/16646/0/page/3
	 * @param context
	 * @param uri
	 * @return
	 * @throws IOException
	 * @throws JSONException
	 */
	public void putData(String uri, JSONObject content) throws IOException {
		// Prepare request (create JSON) 
		String requestBody = content.toString();
		System.out.println(requestBody);
				
		// Open connection
		HttpConnection http = 
			(HttpConnection) Connector.open(uri , Connector.READ_WRITE);
		
		http.setRequestMethod(HttpConnection.POST);
		http.setRequestProperty("Content-Type", "application/json");
		http.setRequestProperty("version", "1.1.0");
		http.setRequestProperty("host", "uni-mannheim.de");
		
		
		// Send request
		DataOutputStream dos = http.openDataOutputStream();  
		byte[] byteRequest = requestBody.getBytes(); 
		
		for(int i = 0; i<byteRequest.length; i++) 
		       dos.writeByte(byteRequest[i]);

		//dos.flush(); //Including this line may produce undesiredresults on certain devices
		/* If you call c.flush(), some devices will send the request in HTTP 1.1 "chunked" encoding by adding a "Transfer-encoding: chunked" header to the request and writing the size of each block of data in hexadecimal form as a text line before the data bytes themselves. This is repeated for any number of blocks you send, usually demarcated by calls to flush().
		If you have a HTTP 1.1-compliant Web server this should be transparent to your server side scripts, but if you do not, you will either want to avoid calling flush() (though some devices may still chunk for you automatically) or write some HTTP 1.1 chunk handling code in your server scripts. All you will need to do in the latter case is read the first text line of the response, extract the size from it (i.e. "C\r\n" where C means 12 bytes of chunk follow), and then read 12 bytes of response data. Then read another line to see if there is another chunk size, and repeat until stream is closed or your script has all the data it needs. See the HTTP 1.1 specification for more details.  */
				
		// Receive answer
		StringBuffer responseBody = new StringBuffer();		
		InputStream dis = http.openInputStream();
		
		int ch;  
		while ((ch = dis.read()) != -1)
			responseBody.append((char) ch);

		System.out.println("Respond: " + responseBody.toString());

		
		// Close connection
		dos.close();
		dis.close();
		http.close(); 

	}
	

}
