package de.uni_mannheim.aware.util;

import java.util.Date;
import java.util.Enumeration;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Vector;

import javax.microedition.lcdui.CustomItem;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Form;

import de.uni_mannheim.aware.api.bluetooth.BluetoothDevice;
import de.uni_mannheim.aware.api.wireless.WirelessAccessPoint;

/**
 * 
 * @author Philipp Heckel
 */
public class TimedList {
	private static final int DEFAULT_ACCURACY = 200;
	public static void main(String[] args) throws InterruptedException {
		TimedList tl = new TimedList();
		
		tl.addExpiryListener(new ExpiryListener() {			
			public void objectsExpired(Vector expiredObjects) {
				System.out.println("EXPIRED: "+expiredObjects);
			}
		});
		
		int i=0;
		while (i++ < 3) {
		tl.replace(new WirelessAccessPoint(new byte[] { 0x00, 0x11}), 1100);
		tl.dump();
		Thread.sleep(1000);
		}
		
		tl.replace(new WirelessAccessPoint(new byte[] { 0x00, 0x22}), 1100);
		Thread.sleep(2000);
		tl.dump();

		
		tl.addExpiryListener(new ExpiryListener() {			
			public void objectsExpired(Vector expiredObjects) {
				System.out.println("e "+expiredObjects);				
			}
		});

		
		/*tl.replace(new WirelessAccessPoint(new byte[] { 0x00, 0x11}), -1100);
		tl.replace(new WirelessAccessPoint(new byte[] { 0x00, 0x12}), 151);
		tl.replace(new String("three"), 1);
		tl.replace(new String("one"), 2);
		tl.replace(new String("three"), 1);
		tl.replace(new String("three"), 1);
		tl.replace(new String("three"), 1);
		Thread.sleep(200);
		tl.replace(new String("three"), 200);
		Thread.sleep(200);
		tl.replace(new String("three"), 200);
		tl.replace(new String("two"), 1102);
		tl.replace(new String("drei vier fünf"), 5000);
		tl.replace(new String("abc"), 1200);
		tl.replace(new String("def"), 5400);
		tl.replace(new String("adsad"), 21100);*/
		
		//tl.dump();
		
		
		
	}
	
	
	/**
	 * Sorted by the ...
	 */
	private Vector sortedList;
	private int accuracy;
	
	private Timer expiryTimer;
	private TimerTask expiryTimerTask;
	private Date expiryTimerExpiryDate;
	
	private Vector listeners;
	
	public TimedList() {
		this(DEFAULT_ACCURACY);		
	}
	
	public TimedList(int accuracy) {
		this.sortedList = new Vector();
		this.accuracy = accuracy;
		
		this.expiryTimer = new Timer();
		this.expiryTimerTask = null;
		this.expiryTimerExpiryDate = null;
		
		this.listeners = new Vector();
		
		
		// new expirytimer
		expiryTimerTask = new TimerTask() { public void run() { expiryTimerTask(); } };
		expiryTimer.schedule(expiryTimerTask, 1000, 1000);
	}
	
	public synchronized void addExpiryListener(ExpiryListener listener) {
		listeners.addElement(listener);
	}
	
	public synchronized void removeExpiryListener(ExpiryListener listener) {
		listeners.removeElement(listener);
	}
	
	/**
	 * 
	 * @param obj Object to add/update to the list
	 * @param TTL Time to live in milliseconds
	 * @return Expiry date, or NULL if given expiry date is in the past
	 */
	public void replace(Object obj, int TTL) {
		replace(obj, new Date(System.currentTimeMillis()+TTL));
	}
	
	/**
	 * 
	 * @param obj Object to add/update to the list
	 * @param expiryDate Expiry date of the entry
	 * @return Expiry date, or NULL if given expiry date is in the past
	 */
	public synchronized void replace(Object obj, Date expiryDate) {
		if (expiryDate.getTime() < System.currentTimeMillis())
			return;

		// Find out if object is already in the list
		synchronized (sortedList) {
			TimedListObject newObj = new TimedListObject(obj, expiryDate);
		
			int currentIndex = sortedList.indexOf(newObj);
			
			// Remove and re-add if found  
			if (currentIndex != -1) {
				//((Form) Display.getDisplay(TestMidlet3.instance).getCurrent()).append("update element "+newObj.getObject()+ " / exp "+newObj.getExpiryDate()+"\n");

				try { sortedList.removeElementAt(currentIndex); }
				catch (ArrayIndexOutOfBoundsException e) { 
					//((Form) Display.getDisplay(TestMidlet3.instance).getCurrent()).append("ERR array out of bound "+e+"\n");
				}
				replace(obj, expiryDate);
				
				return;
			}
			
			// Add if not found
			//((Form) Display.getDisplay(TestMidlet3.instance).getCurrent()).append("add elem "+newObj.getObject()+"\n");
			
			int addIndex = -1;
			
			for (int i = 0; i<sortedList.size(); i++) {
				TimedListObject listObj = (TimedListObject) sortedList.elementAt(i);
				
				if (listObj.getExpiryDate().getTime() < newObj.getExpiryDate().getTime())
					continue;
				
				addIndex = i;
				break;
			}
			
			if (addIndex == -1)
				sortedList.addElement(newObj);
			else
				sortedList.insertElementAt(newObj, addIndex);			
						
		}

		// Cancel timer and reset it, if fire date has changed
		resetTimer();
	}
	

	public synchronized boolean contains(Object obj) {
		return sortedList.indexOf(obj) != -1;
	}
	
	public synchronized Vector getObjects() {
		Vector objects = new Vector();
		
		for (Enumeration e = sortedList.elements(); e.hasMoreElements(); ) {
			TimedListObject tlo = (TimedListObject) e.nextElement();
			
			if (tlo != null && tlo.getObject() != null)
				objects.addElement(tlo.getObject());
		}
			
		return objects;
	}	
	
	public synchronized boolean remove(Object obj) {
		return sortedList.removeElement(obj);
	}
	
	public synchronized void removeAllElements() {
		sortedList.removeAllElements();
	}
	
	public String dump() {
		String s = "";
		for (int i= 0; i<sortedList.size(); i++) {
			TimedListObject tlo = (TimedListObject)sortedList.elementAt(i);
			s += "obj = "+tlo.getObject()+", expirydate = "+tlo.getExpiryDate()+"\n";

		}
		System.out.println(s);
		return s;
	}
	
	private synchronized void expiryTimerTask() {
		synchronized (sortedList) {
			Vector expiredObjects = new Vector();
			
			while (sortedList.size() > 0) {
				TimedListObject firstObj = (TimedListObject) sortedList.elementAt(0);
				
				// First element not yet expired; stop loop
				if (System.currentTimeMillis()+accuracy < firstObj.getExpiryDate().getTime()) 
					break;
				
				// Add to 'expiryList'; and remove it from the list
				expiredObjects.addElement(firstObj.getObject());
				sortedList.removeElementAt(0);			
			}
			
			// Fire expiry event if list not empty
			if (expiredObjects.size() != 0)
				fireObjectExpiredEvent(expiredObjects);
		}
		
		// Set new timer
		resetTimer();		
	}
	
	private void fireObjectExpiredEvent(Vector expiredObjects) {
		//((Form) Display.getDisplay(TestMidlet3.instance).getCurrent()).append("  EXPIRED: "+expiredObjects+"\n");
		
		for (Enumeration e = listeners.elements(); e.hasMoreElements(); ) {
			ExpiryListener listener = (ExpiryListener) e.nextElement();
			
			if (listener != null)
				listener.objectsExpired(expiredObjects);
		}		
	}

	private synchronized void resetTimer() {
		return;/*
		if (sortedList.size() == 0) {
			((Form) Display.getDisplay(TestMidlet3.instance).getCurrent()).append("reset timer size=0\n");
			return;
		}
		
		((Form) Display.getDisplay(TestMidlet3.instance).getCurrent()).append("reset timer\n");
		TimedListObject firstObj = (TimedListObject) sortedList.elementAt(0);
		
		if (!firstObj.getExpiryDate().equals(expiryTimerExpiryDate)) {
			// Cancel all outstanding tasks
			if (expiryTimerTask != null)
				expiryTimerTask.cancel();
			
			// Create new task and schedule it
			expiryTimerExpiryDate = new Date(firstObj.getExpiryDate().getTime());
			expiryTimerTask = new TimerTask() {
				public void run() { expiryTimerTask(); } };
				
			((Form) Display.getDisplay(TestMidlet3.instance).getCurrent()).append("  next time: "+expiryTimerExpiryDate+"\n");
			expiryTimer.schedule(expiryTimerTask, expiryTimerExpiryDate);
		}*/
	}
	
	public interface ExpiryListener {
		public void objectsExpired(Vector expiredObjects);
	}
	
	private class TimedListObject {
		private Object object;
		private Date expiryDate;
		
		public TimedListObject(Object object, Date expiryDate) {
			this.object = object;
			this.expiryDate = expiryDate;
		}
		
		public void setExpiryDate(Date expiryDate) {
			this.expiryDate = expiryDate;
		}
		
		public Date getExpiryDate() {
			return expiryDate;
		}
		
		public void setObject(Object object) {
			this.object = object;
		}
		
		public Object getObject() {
			return object;
		}
		
		/* @Override */
		public boolean equals(Object obj) {
			if (obj == null || !(obj instanceof TimedListObject))
				return false;
			
			TimedListObject tlo = (TimedListObject) obj;
			
			return tlo.getObject() != null 
				&& tlo.getObject().equals(getObject());
		}
	}

}
