package de.uni_mannheim.aware.util;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import javax.bluetooth.RemoteDevice;

/**
 * 
 * @author Philipp Heckel
 */
public class VectorUtil {
	/**
	 * A \ B
	 */
	public static Vector difference(Vector A, Vector B) {
		Vector result = new Vector();
		
		for (Enumeration en = A.elements(); en.hasMoreElements(); ) { 
			Object obj = en.nextElement();

			if (!B.contains(obj))
				result.addElement(obj);
		}
		
		return result;
	}
	
	/**
	 * A ∩ B
	 */
	public static Vector intersection(Vector A, Vector B) {
		Vector result = new Vector();
		
		for (Enumeration en = A.elements(); en.hasMoreElements(); ) { 
			Object obj = en.nextElement();

			if (B.contains(obj))
				result.addElement(obj);
		}
		
		return result;
	}	
	
	/**
	 * A ∪ B
	 */
	public static Vector union(Vector A, Vector B) {
		Vector result = new Vector();
		
		for (Enumeration en = A.elements(); en.hasMoreElements(); ) 
			result.addElement(en.nextElement());
		
		for (Enumeration en = B.elements(); en.hasMoreElements(); ) {
			Object obj = en.nextElement();
			
			if (!result.contains(obj))
				result.addElement(obj);	
		}
		
		return result;
	}	
	
	public static Vector fromArray(Object[] array) {
		Vector result = new Vector();
		
		for (int i = 0; i<array.length; i++)
			result.addElement(array[i]);
		
		return result;
	}
	
	public static Object[] toArray(Vector vector) {
		Object[] result = new Object[vector.size()];
		
		for (int i = 0; i<result.length; i++)
			result[i] = vector.elementAt(i);
		
		return result;		
	}
	
	public static Vector clone(Vector vector) {
		return fromArray(toArray(vector));
	}
	
	/**
	 * Like {@link Vector#equals(Object)}, but ignores the order of the elements.
	 * Returns true if vector A contains the same elements as vector B.
	 * 
	 * <p><b>Warning:</b> In the worst case, this function has a complexity 
	 * of O(n²).
	 *  
	 * @param a First vector
	 * @param b Second vector
	 * 
	 * @return <em>true</em> if both vectors contain the same elements.
	 */
	public static boolean equalElements(Vector a, Vector b) {
		if (a.size() != b.size())
			return false;
		
		for(Enumeration ea = a.elements(); ea.hasMoreElements(); ) {
			if (!b.contains(ea.nextElement()))
				return false;
		}
		
		return true;
	}
	
	public static void main(String[] args) {
		Vector A = new Vector();
		Vector B = new Vector();
		
		A.addElement("a");
		A.addElement("b");
		A.addElement("c");
		A.addElement("d");
		
		B.addElement("d");		
		B.addElement("X");
		B.addElement("b");
		B.addElement("c");

		System.out.println(equalElements(A, B));
		
		System.out.println("A = "+A);		
		System.out.println("B = "+B);

		System.out.println("A \\ B = "+difference(A, B));
		System.out.println("B \\ A = "+difference(B, A));
		System.out.println("B ∩ A = "+intersection(B, A));
		System.out.println("B ∪ A = "+union(B, A));
	}
}
