package de.uni_mannheim.aware.api.location;

import de.uni_mannheim.aware.api.FuzzyCondition;
import de.uni_mannheim.aware.api.SensorData;
import de.uni_mannheim.aware.api.SensorFactory;
import de.uni_mannheim.aware.api.SensorListener;
import de.uni_mannheim.aware.impl.SensorNotAvailableException;
import de.uni_mannheim.aware.impl.UnsupportedPlatformException;

public abstract class FuzzyLocationCondition extends FuzzyCondition implements SensorListener {
	public FuzzyLocationCondition() throws UnsupportedPlatformException, SensorNotAvailableException {
		super(SensorFactory.getImpl().getLocationSensor());
		
		setSensorData(new LocationData());
	}
	
	public LocationData getLocationData() {
		return (LocationData) getSensorData();
	}
	
	/* @Override */
	public void sensorDataChanged(SensorData sensorData) {
		if (! (sensorData instanceof LocationData))
			return;
				
		// Update data
		LocationData data = (LocationData) sensorData;

		if (data.equals(getLocationData()))
			return;

		setSensorData(data);

		// Check changes
		double newScore = score(data);
		
		// Notify if changed
		if (Math.abs(score - newScore) < 0.01) {
			setScore(newScore);	
			fireConditionChangedEvent();
		}		
	}
	
	protected abstract double score(LocationData currentLocation);	
}